# Copyright (c) 2021 aerocyber
# 
# This software is released under the MIT License.
# https://opensource.org/licenses/MIT

"""
   Manage osmations: add, delete, edit, view.
"""

def addOsmation(db: list, name: str, url: str, cat=None):
    """Add osmation to the included osmations.

    Args:
        db (list): List of osmations.
        name (str): Name associated with the url.
        url (str): Url associated with the name.
        cat ((None OR list), optional): Categories to which the osmation belongs to. Defaults to None.

    Raises:
        TypeError: Invalid type.

    Returns:
        list: List of osmations.
    """
    dat = {
        "Name": name,
        "URL": url
    }
    if cat != None:
        if type(cat) is not list:
            raise TypeError("{cat} is expected to be of type list.".format(
                cat=cat
            ))
        dat["Categories"] = cat
    db.append(dat)
    return db

def delOsmation(db: list, name: str):
    """Delete an osmation with the name as given.

    Args:
        db (list): List of osmations.
        name (str): Name of osmation.

    Returns:
        db: List of osmations with name removed.
    """
    for i in db:
        if name == i["Name"]:
            _ = db.pop(db.index(i))
            break
    return db

def modName(name: str, db: list, newName: str):
    """Change the name of osmation.

    Args:
        name (str): Old name of osmation.
        db (list): List of osmations.
        newName (str): New name of osmation.

    """
    for i in db:
        if i["Name"] == name:
            i["Name"] = newName
            break


def changeURL(name: str, newUrl: str, db: str):
    """Change URL of osmation.

    Args:
        name (str): Name to which the new url is to be associated with.
        newUrl (str): [description]
        db (list): List of osmations.
    """
    for i in db:
        if i["Name"] == name:
            i["URL"] = newUrl
            break

def remByName(name: str, db: list):
    """Delete osmation by name

    Args:
        name (str): Name of osmation to delete.
        db (list): List of osmations.
    """
    for i in db:
        if i["Name"] == name:
            index = db.index(i)
            _ = i.pop(index)


# Copyright (c) 2021 aerocyber
# 
# This software is released under the MIT License.
# https://opensource.org/licenses/MIT

from . import loadOMIO, manageOsmation, writeOMIO, __version__

__version__ = __version__.__version__


# Copyright (c) 2021 aerocyber
# 
# This software is released under the MIT License.
# https://opensource.org/licenses/MIT

import os
import json
import hashlib

"""Decode and decrypt .omio file which contain osmations."""

class InValidOMIO(Exception):
    """Exception raised if the .omio file is invalid.

    Args:
        Exception : file path and message.
    """
    
    def __init__(self, file, message="The omio file does not meet the osmata guidelines."):
        self.file =  file
        self.message = message
        super().__init__(self.message)

    def __str__(self):
        return f'{self.file} -> {self.message}'

def validateAndOpen(OMIO_Path):
    """validate OMIO file and if valid, return data and header.

    Args:
        OMIO_Path (str): Path to .omio file.

    Raises:
        InValidOMIO: Exception if the omio file is not valid.
        
    Returns:
        header (dict): Information stored in the OMIO file.
        data (list): The osmations!
    """
    file = os.path.normpath(
            os.path.normcase(
                OMIO_Path
            )
        ) # Normalize path to omio file.
    f = open(
        file, 'r'
    )
    dat = json.load(f)
    f.close()
    if type(dat) is not dict: # Make sure the type of dat is dict
        raise InValidOMIO(file=file, message="The type of data MUST be dict.")
    data = dat["Data"]
    if type(data) is not list:
        raise InValidOMIO(file=file, message="Data in the key Data must be a list of dict.")
    for i in data:
        if type(i) is not dict:
            raise InValidOMIO(file=file, message="Each data INSIDE the Data list MUST be of type dict")
    try:
        header = dat["Header"]
    except KeyError():
        raise InValidOMIO(file=file, message="Header for the .omio file is missing...")
    else:
        sha256Data = hashlib.sha256(data)
        if sha256Data != header["SHA-256"]:
            raise InValidOMIO(file=file, message="Data Integrity check failed.")
        return header, data



# Copyright (c) 2021 aerocyber
# 
# This software is released under the MIT License.
# https://opensource.org/licenses/MIT

import os
import json
import hashlib

"""Write OMIO files."""

def createOMIO(db, outfile, header=None):
    """Create OMIO in a standard way.

    Args:
        db (list): Database: List of dict.
        outfile (str): Path to file to which the osmations are written. If file exist, it is rewritten.
        header (dict, optional): Additional data to be included in header. Defaults to None.

    Raises:
        TypeError: For every invalid types at the required positions.

    """
    if type(db) is not list:
        raise TypeError("{db} is expected to be of type list.".format(db=db))
    for i in db:
        if type(i) is not dict:
            raise TypeError("{i} in {db} is expected to be of type dict".format(
                i = i,
                db = db
            ))
    headers = {}
    if header != None:
        if type(header) is not dict:
            raise TypeError("{header} is expected to be None or of type dict.".format(
                header=header
            ))
        headers.update(header)
    headers["SHA-256"] = hashlib.sha256(db) # For ospyata and other osmation bindings to validate data integrity.
    data = {"Data": db}
    omioFormat = {}
    omioFormat.update(headers)
    omioFormat.update(data)
    file = os.path.normcase(
        os.path.normpath(
            outfile
        )
    )
    try:
        f = open(file, 'w')
        f.write(json.dumps(omioFormat))
        f.close()
    except Exception as e:
        raise e



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['ospyata']

package_data = \
{'': ['*']}

install_requires = \
['pycryptodome>=3.10.1,<4.0.0',
 'requests>=2.26.0,<3.0.0',
 'shortlink>=0.1.6,<0.2.0']

setup_kwargs = {
    'name': 'ospyata',
    'version': '1.0.0',
    'description': 'Python bindings for osmata.',
    'long_description': None,
    'author': 'aerocyber',
    'author_email': None,
    'maintainer': None,
    'maintainer_email': None,
    'url': None,
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'python_requires': '>=3.7,<4.0',
}


setup(**setup_kwargs)
